# marvin.tcl
# written by Chad Wagner <cmwagner@sodre.net>
#
# 08-12-96 cmwagner@sodre.net Wrote this for someone who wanted a wacky
#                             bot that went into channels at random and
#                             said a witty phrase.
#
# This script will say a witty phrase to multiple channels, and even
# join/part the channels if the bot is not normally on them.  All
# configuration should be done in the marvin.cfg file, nothing else
# needs to be configured here.  Original idea by Raistlin, for his
# bot Marvin the Paranoid Android.

# check to make sure that toolbox is loaded
if {![info exists toolbox_loaded]} {
   putlog "marvin.tcl: toolbox.tcl is required for this script."
   return 0
}

# main procedure, join channel say witty phrase then bolt
proc m_marvinize_channel {} {
   global m_hatch m_phrases m_channels m_debug

   # grab channel and phrase randomly from lists
   set chan [lindex $m_channels [rand [llength $m_channels]]]
   set phrase [lindex $m_phrases [rand [llength $m_phrases]]]

   # if bot is already on the channel, then don't join/part
   set botonchan 0
   foreach j [channels] {
      if {$chan == $j} {
         set botonchan 1
      }
   }

   # say what I got to say... if I am not on the channel join and then
   # bolt a minute later...
   if {$botonchan == 1} {
      putserv "PRIVMSG $chan :$phrase"
   } {
      channel add $chan
      channel set $chan -clearbans -enforcebans +dynamicbans +userbans -autoop -bitch -greet -protectops -statuslog -stopnethack -revenge
      putserv "PRIVMSG $chan :$phrase"
      timer 1 "catch {channel remove $chan}"
   }

   if {[info exists m_debug]} {
      putlog "m_marvinize_channel:$chan $botonchan $phrase"
   }

   # let's keep this rollin... setup another timer to loop back in in
   # m_hatch minutes
   timer $m_hatch m_marvinize_channel
}

# initialize variables procedure
proc m_init {} {
   global m_config m_hatch m_phrases m_channels

   # configuration file -- where pertinent data is stored for later retrieval
   if {![info exists m_config]} {
      set m_config "marvin.cfg"
   }

   # hatch rate -- how often it hatches a message to a channel
   if {![info exists m_hatch]} {
      set m_hatch 30
   }

   # phrases list (string -- can just wipe it out)
   set m_phrases ""

   # channels list (string -- can just wipe it out)
   set m_channels ""

}

# load in configuration
proc m_config_load {} {
   global m_config m_hatch m_phrases m_channels m_debug

   # 1 if couldn't open
   if {[catch {set fd [open $m_config r]}] != 0} { return 1 }

   # data gathering loop, keep looping until at eof
   while {![eof $fd]} {
      # get a line of data
      set inp [gets $fd]
      # if eof, break out of loop
      if {[eof $fd]} { break }
      # if it is nothing but spaces, skip it and grab another line
      if {[string trim $inp " "] == ""} { continue }

      # it's a comment, skip it
      if {[string index $inp 0] == "#"} { continue }

      # if debugging is enabled, log it
      if {[info exists m_debug]} {
         putlog "m_config_load:$inp"
      }

      # get configuration command
      set cmd [lindex $inp 0]

      # add channel to channels list
      if {$cmd == "C"} {
         # going to foreach a possible multiple C command because we don't
         # want to append the entire range and end up having it mistaken
         # "#lamest #twilight_zone" as an entire channel name, would be
         # bad, so this seems to work... :)
         foreach j [lrange $inp 1 end] {
            lappend m_channels $j
         }
      }
      # add phrase to phrase list
      if {$cmd == "P"} {
         lappend m_phrases [lrange $inp 1 end]
      }

      # change hatch rate
      if {$cmd == "H"} {
         set m_hatch [lindex $inp 1]
         if {$m_hatch < 5} {
            putlog "marvin.tcl: Setting hatch rate to $m_hatch -- if I get confused it is your fault :)"
         }
      }
   }

   # no errors
   return 0
}

# initialize variables
m_init

# load configuration, if returns 1 then quit, don't fire up the timer
if {[m_config_load] == 1} {
   putlog "marvin.tcl: error while loading configuration file."
   return 0
}

# start the timer -- check to make sure it is not running already :)
if {[timerexists "m_marvinize_channel"] == ""} {
   m_marvinize_channel
}

putlog "marvin.tcl loaded."

if {[info exists m_debug]} {
   putlog "marvin.tcl: m_hatch=$m_hatch initial timer: [timerexists "m_marvinize_channel"]"
}
